@@ -54,14 +54,20 @@ from sqlalchemy import VARCHAR
 from sqlalchemy.dialects.mysql import base as mysql
 from sqlalchemy.dialects.mysql import insert
 from sqlalchemy.dialects.mysql import match
+from sqlalchemy.orm import DeclarativeBase
+from sqlalchemy.orm import Mapped
+from sqlalchemy.orm import mapped_column
 from sqlalchemy.sql import column
+from sqlalchemy.sql import delete
 from sqlalchemy.sql import table
+from sqlalchemy.sql import update
 from sqlalchemy.sql.expression import bindparam
 from sqlalchemy.sql.expression import literal_column
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import eq_ignore_whitespace
+from sqlalchemy.testing import expect_raises
 from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import mock
@@ -432,6 +438,21 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
                 "description", String(255), server_default=func.lower("hi")
             ),
             Column("data", JSON, server_default=func.json_object()),
+            Column(
+                "updated1",
+                DateTime,
+                server_default=text("now() on update now()"),
+            ),
+            Column(
+                "updated2",
+                DateTime,
+                server_default=text("now() On  UpDate now()"),
+            ),
+            Column(
+                "updated3",
+                DateTime,
+                server_default=text("now() ON UPDATE now()"),
+            ),
         )
 
         eq_(dialect._support_default_function, has_brackets)
@@ -440,10 +461,13 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
             self.assert_compile(
                 schema.CreateTable(tbl),
                 "CREATE TABLE testtbl ("
-                "time DATETIME DEFAULT (CURRENT_TIMESTAMP), "
+                "time DATETIME DEFAULT CURRENT_TIMESTAMP, "
                 "name VARCHAR(255) DEFAULT 'some str', "
                 "description VARCHAR(255) DEFAULT (lower('hi')), "
-                "data JSON DEFAULT (json_object()))",
+                "data JSON DEFAULT (json_object()), "
+                "updated1 DATETIME DEFAULT now() on update now(), "
+                "updated2 DATETIME DEFAULT now() On  UpDate now(), "
+                "updated3 DATETIME DEFAULT now() ON UPDATE now())",
                 dialect=dialect,
             )
         else:
@@ -453,7 +477,10 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
                 "time DATETIME DEFAULT CURRENT_TIMESTAMP, "
                 "name VARCHAR(255) DEFAULT 'some str', "
                 "description VARCHAR(255) DEFAULT lower('hi'), "
-                "data JSON DEFAULT json_object())",
+                "data JSON DEFAULT json_object(), "
+                "updated1 DATETIME DEFAULT now() on update now(), "
+                "updated2 DATETIME DEFAULT now() On  UpDate now(), "
+                "updated3 DATETIME DEFAULT now() ON UPDATE now())",
                 dialect=dialect,
             )
 
@@ -724,6 +751,14 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             .with_dialect_options(mysql_limit=5),
             "UPDATE t SET col1=%s LIMIT 5",
         )
+
+        # does not make sense but we want this to compile
+        self.assert_compile(
+            t.update()
+            .values({"col1": 123})
+            .with_dialect_options(mysql_limit=0),
+            "UPDATE t SET col1=%s LIMIT 0",
+        )
         self.assert_compile(
             t.update()
             .values({"col1": 123})
@@ -738,6 +773,39 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             "UPDATE t SET col1=%s WHERE t.col2 = %s LIMIT 1",
         )
 
+    def test_delete_limit(self):
+        t = sql.table("t", sql.column("col1"), sql.column("col2"))
+
+        self.assert_compile(t.delete(), "DELETE FROM t")
+        self.assert_compile(
+            t.delete().with_dialect_options(mysql_limit=5),
+            "DELETE FROM t LIMIT 5",
+        )
+        # does not make sense but we want this to compile
+        self.assert_compile(
+            t.delete().with_dialect_options(mysql_limit=0),
+            "DELETE FROM t LIMIT 0",
+        )
+        self.assert_compile(
+            t.delete().with_dialect_options(mysql_limit=None),
+            "DELETE FROM t",
+        )
+        self.assert_compile(
+            t.delete()
+            .where(t.c.col2 == 456)
+            .with_dialect_options(mysql_limit=1),
+            "DELETE FROM t WHERE t.col2 = %s LIMIT 1",
+        )
+
+    @testing.combinations((update,), (delete,))
+    def test_update_delete_limit_int_only(self, crud_fn):
+        t = sql.table("t", sql.column("col1"), sql.column("col2"))
+
+        with expect_raises(ValueError):
+            crud_fn(t).with_dialect_options(mysql_limit="not an int").compile(
+                dialect=mysql.dialect()
+            )
+
     def test_utc_timestamp(self):
         self.assert_compile(func.utc_timestamp(), "utc_timestamp()")
 
@@ -1300,6 +1368,25 @@ class InsertOnDuplicateTest(fixtures.TestBase, AssertsCompiledSQL):
             dialect=dialect,
         )
 
+    def test_on_update_instrumented_attribute_dict(self):
+        class Base(DeclarativeBase):
+            pass
+
+        class T(Base):
+            __tablename__ = "table"
+
+            foo: Mapped[int] = mapped_column(Integer, primary_key=True)
+
+        q = insert(T).values(foo=1).on_duplicate_key_update({T.foo: 2})
+        self.assert_compile(
+            q,
+            (
+                "INSERT INTO `table` (foo) VALUES (%s) "
+                "ON DUPLICATE KEY UPDATE foo = %s"
+            ),
+            {"foo": 1, "param_1": 2},
+        )
+
 
 class RegexpCommon(testing.AssertsCompiledSQL):
     def setup_test(self):
